(* ::Package:: *)

(*:Package Version: 3.06 *)

(*:Mathematica Version: 8.01 *)

(*:Version History: 1.01 (2001.08.01 - 2013.09.21)
                    2.01 (2013.09.22 - 2020.12.03)
                    3.01 (2020.12.04 - 2021.11.22)Legendre-Cos, Toroidal, Conical
                    3.02 (2021.11.23 - 2022.02.05)NistChebyshev
                    3.03 (2022.02.06 - 2022.05.05)Gegenbauer
                    3.04 (2022.05.06 - 2023.01.30)UltraSphericalHarmonicY
                    3.05 (2023.01.31 - 2023.04.01)Jacobi, Romanovski, Zernike
                    3.06 (2023.04.02 - 20XX.XX.XX)JacobiPCos, WignerD *)

(*:Copyright: Copyright 2001.08-2023.04, Souichiro Ikebe *)

(*:Author: Souichiro Ikebe *)

(*:Name: SpecialFunctions`GaussHypergeometric` *)

(*:Title: GaussHypergeometric *)

(*:Keywords: LegendreQ2, LegendrePCos, LegendreQCos, 
   LegendrePCosAC, LegendreQCosAC, 
   LegendrePCosh, LegendreQCosh, LegendrePCoshAC, LegendreQCoshAC, 
   ToroidalP, ToroidalQ, ConicalP, ConicalQ, 
   ConicalPAC, ConicalQAC, NistConicalP, NistConicalQ, 
   ChebyshevV, ChebyshevTNormalized, ChebyshevUNormalized, 
   ChebyshevVNormalized, NistChebyshevV, NistChebyshevW, 
   GegenbauerS, GegenbauerD, GegenbauerCNormalized, 
   GegenbauerSNormalized, UltraSphericalHarmonicY, 
   JacobiG, JacobiQ, JacobiPNormalized, JacobiQNormalized, 
   JacobiQFerrers, JacobiQHobson, RomanovskiR, ZernikeZ, 
   JacobiPCos, WignerDOrdinary, WignerDOrdinaryAnalytic, WignerDAnalytic, 
   Hypergeometric2G1, RiemannHypergeometricP, 
   RiemannHypergeometricQ *)

(*:Summary:
    This package implements functions of relation 
    Gauss's 2F1-Hypergeometric functions. *)

(************************************************************************)

BeginPackage["SpecialFunctions`GaussHypergeometric`"]



LegendreQ2::usage =
"LegendreQ2[nu,mu,a,z] gives the Legendre functions of second
 kind of original scheme : 2/Pi*LegendreQ[nu,mu,a,z].
 LegendreQ2[nu,mu,2,z] is Ferrers's type. and
 LegendreQ2[nu,mu,3,z] is Hobson's type."

LegendrePCos::usage =
"LegendrePCos[nu,mu,a,th] gives the analytic continuation
 of Legendre functions of first kind : P(nu,mu,a,cos(th)).
 a=1 or a=2 is Ferrers's type, and a=3 is Hobson's type.
 LegendrePCos[nu,mu,th]=LegendrePCos[nu,mu,2,th],
 and LegendrePCos[nu,th]=LegendrePCos[nu,0,2,th]."

LegendreQCos::usage =
"LegendreQCos[nu,mu,a,th] gives the analytic continuation
 of Legendre functions of second kind : Q(nu,mu,a,cos(th)).
 a=1 or a=2 is Ferrers's type, and a=3 is Hobson's type.
 LegendreQCos[nu,mu,th]=LegendreQCos[nu,mu,2,th],
 and LegendreQCos[nu,th]=LegendreQCos[nu,0,2,th]."

LegendrePCosAC::usage =
"LegendrePCosAC[nu,mu,3,th] gives the other analytic continuation
 of Legendre functions of first kind : P(nu,mu,3,cos(th)).
 LegendrePCosAC is Hobson's type only.
 LegendrePCosAC[nu,mu,th]=LegendrePCosAC[nu,mu,3,th],
 and LegendrePCosAC[nu,th]=LegendrePCosAC[nu,0,3,th]."

LegendreQCosAC::usage =
"LegendreQCosAC[nu,mu,3,th] gives the other analytic continuation
 of Legendre functions of second kind : Q(nu,mu,3,cos(th)).
 LegendreQCosAC is Hobson's type only.
 LegendreQCosAC[nu,mu,th]=LegendreQCosAC[nu,mu,3,th],
 and LegendreQCosAC[nu,th]=LegendreQCosAC[nu,0,3,th]."

LegendrePCosh::usage =
"LegendrePCosh[nu,mu,3,th] gives the analytic continuation
 of Legendre functions of first kind : P(nu,mu,3,cosh(th)).
 LegendrePCosh is Hobson's type only.
 LegendrePCosh[nu,mu,th]=LegendrePCosh[nu,mu,3,th],
 and LegendrePCosh[nu,th]=LegendrePCosh[nu,0,3,th]."

LegendreQCosh::usage =
"LegendreQCosh[nu,mu,3,th] gives the analytic continuation
 of Legendre functions of second kind : Q(nu,mu,3,cosh(th)).
 LegendreQCosh is Hobson's type only.
 LegendreQCosh[nu,mu,th]=LegendreQCosh[nu,mu,3,th],
 and LegendreQCosh[nu,th]=LegendreQCosh[nu,0,3,th]."

LegendrePCoshAC::usage =
"LegendrePCoshAC[nu,mu,3,th] gives the other analytic continuation
 of Legendre functions of first kind : P(nu,mu,3,cosh(th)).
 LegendrePCoshAC is Hobson's type only.
 LegendrePCoshAC[nu,mu,th]=LegendrePCoshAC[nu,mu,3,th],
 and LegendrePCoshAC[nu,th]=LegendrePCoshAC[nu,0,3,th]."

LegendreQCoshAC::usage =
"LegendreQCoshAC[nu,mu,3,th] gives the other analytic continuation
 of Legendre functions of second kind : Q(nu,mu,3,cosh(th)).
 LegendreQCoshAC is Hobson's type only.
 LegendreQCoshAC[nu,mu,th]=LegendreQCoshAC[nu,mu,3,th],
 and LegendreQCoshAC[nu,th]=LegendreQCoshAC[nu,0,3,th]."

ToroidalP::usage =
"ToroidalP[nu,mu,a,z] gives the Toroidal functions of first kind.
 a=1 or a=2 is Ferrers's type : P(nu-1/2,mu,2,cos(z)).
 and a=3 is Hobson's type : P(nu-1/2,mu,3,cosh(z)).
 ToroidalP[nu,mu,z]=ToroidalP[nu,mu,3,z],
 and ToroidalP[nu,z]=ToroidalP[nu,0,3,z]."

ToroidalQ::usage =
"ToroidalQ[nu,mu,a,z] gives the Toroidal functions of second kind.
 a=1 or a=2 is Ferrers's type : Q(nu-1/2,mu,2,cos(z)).
 and a=3 is Hobson's type : Q(nu-1/2,mu,3,cosh(z)).
 ToroidalQ[nu,mu,z]=ToroidalQ[nu,mu,3,z],
 and ToroidalQ[nu,z]=ToroidalQ[nu,0,3,z]."

ConicalP::usage =
"ConicalP[nu,mu,a,th] gives the Conical functions of first kind.
 a=1 or a=2 is Ferrers's type : P(-1/2+I*nu,mu,2,cos(th)).
 and a=3 is Hobson's type : P(-1/2+I*nu,mu,3,cos(th)).
 ConicalP[nu,mu,th]=ConicalP[nu,mu,2,th],
 and ConicalP[nu,th]=ConicalP[nu,0,2,th]."

ConicalQ::usage =
"ConicalQ[nu,mu,a,th] gives the Conical functions of second kind.
 a=1 or a=2 is Ferrers's type : Q(-1/2+I*nu,mu,2,cos(th)).
 and a=3 is Hobson's type : Q(-1/2+I*nu,mu,3,cos(th)).
 ConicalQ[nu,mu,th]=ConicalQ[nu,mu,2,th],
 and ConicalQ[nu,th]=ConicalQ[nu,0,2,th]."

ConicalPAC::usage =
"ConicalPAC[nu,mu,3,th] gives the other analytic continuation
 of Conical functions of first kind : P(-1/2+I*nu,mu,3,cos(th)).
 ConicalPAC is Hobson's type only.
 ConicalPAC[nu,mu,th]=ConicalPAC[nu,mu,3,th],
 and ConicalPAC[nu,th]=ConicalPAC[nu,0,3,th]."

ConicalQAC::usage =
"ConicalQAC[nu,mu,3,th] gives the other analytic continuation
 of Conical functions of second kind : Q(-1/2+I*nu,mu,3,cos(th)).
 ConicalQAC is Hobson's type only.
 ConicalQAC[nu,mu,th]=ConicalQAC[nu,mu,3,th],
 and ConicalQAC[nu,th]=ConicalQAC[nu,0,3,th]."

NistConicalP::usage =
"NistConicalP[nu,mu,th] gives the Conical functions of
 first kind defined in NIST(section:14.20).
 NistConicalP is Ferrers's type only.
 NistConicalP[nu,th]=NistConicalP[nu,0,th]."

NistConicalQ::usage =
"NistConicalQ[nu,mu,th] gives the Conical functions of
 second kind defined in NIST(section:14.20).
 NistConicalQ is Ferrers's type only.
 NistConicalQ[nu,th]=NistConicalQ[nu,0,th]."

ChebyshevV::usage =
"ChebyshevV[v,z] gives the Chebyshev's functions
 of second kind."

ChebyshevTNormalized::usage =
"ChebyshevTNormalized[v,z] gives the normalized
 Chebyshev's functions of first kind."

ChebyshevUNormalized::usage =
"ChebyshevUNormalized[v,z] gives the normalized
 Chebyshev's functions of second kind."

ChebyshevVNormalized::usage =
"ChebyshevVNormalized[v,z] gives the normalized
 Chebyshev's functions of second kind."

NistChebyshevV::usage =
"NistChebyshevV[v,z] gives the Chebyshev's V-functions
 defined in NIST(section:18.5)."

NistChebyshevW::usage =
"NistChebyshevW[v,z] gives the Chebyshev's W-functions
 defined in NIST(section:18.5)."

GegenbauerS::usage =
"GegenbauerS[v,w,z] gives the Gegenbauer's functions
 of second kind (Ferrers type)."

GegenbauerD::usage =
"GegenbauerD[v,w,z] gives the Gegenbauer's functions
 of second kind (Hobson type)."

GegenbauerCNormalized::usage =
"GegenbauerCNormalized[v,w,z] gives the normalized
 Gegenbauer's functions of first kind."

GegenbauerSNormalized::usage =
"GegenbauerSNormalized[v,w,z] gives the normalized
 Gegenbauer's functions of second kind."

UltraSphericalHarmonicY::usage =
"UltraSphericalHarmonicY[{n[1],...,n[d-1]},{t[1],...,t[d-1]}] gives
 the Ultra spherical harmonics of d-dimensions."

JacobiG::usage =
"JacobiG[v,p,q,z] gives the Jacobi's functions of first kind
 of other type :
 JacobiG[v,p,q,z]=JacobiP[v,p-q,q-1,2*z-1]/Binomial[2*v+p-1,v]."

JacobiQ::usage =
"JacobiQ[v,a,b,z] gives the Jacobi's functions of second kind."

JacobiPNormalized::usage =
"JacobiPNormalized[v,a,b,z] gives the normalized
 Jacobi's functions of first kind."

JacobiQNormalized::usage =
"JacobiQNormalized[v,a,b,z] gives the normalized
 Jacobi's functions of second kind."

JacobiQFerrers::usage =
"JacobiQFerrers[v,a,b,z] gives the Jacobi's functions of second kind
 of Ferrers's type defined in Higher Trans. Func.(Vol.2), pp.171."

JacobiQHobson::usage =
"JacobiQHobson[v,a,b,z] gives the Jacobi's functions of second kind
 of Hobson's type defined in Higher Trans. Func.(Vol.2), pp.170."

RomanovskiR::usage =
"RomanovskiR[v,a,b,z] gives the Romanovski's R-functions(polynomials)."

ZernikeZ::usage =
"ZernikeZ[n,m,rho,phi] gives the Zernike eigen functions."

JacobiPCos::usage =
"JacobiPCos[nu,a,b,th] gives the analytic continuation
 of Jacobi functions of first kind : P(nu,a,b,cos(th))."

WignerDOrdinary::usage =
"WignerDOrdinary[{j,m1,m2},ps,th,ph] gives the Wigner's D-functions
 in ordinary definition (= Non Mathematica definition).
 WignerDOrdinary[{j,m1,m2},th] gives the Wigner's d-functions
 in ordinary definition (= Non Mathematica definition)."

WignerDOrdinaryAnalytic::usage =
"WignerDOrdinaryAnalytic[{j,m1,m2},ps,th,ph] gives the analytic
 continuation of Wigner's D-functions in ordinary definition.
 WignerDOrdinaryAnalytic[{j,m1,m2},th] gives the analytic
 continuation of Wigner's d-functions in ordinary definition."

WignerDAnalytic::usage =
"WignerDAnalytic[{j,m1,m2},ps,th,ph] gives the analytic continuation
 of Wigner's D-functions in Mathematica definition.
 WignerDAnalytic[{j,m1,m2},th] gives the analytic continuation
 of Wigner's d-functions in Mathematica definition."

Hypergeometric2G1::usage =
"Hypergeometric2G1[a,b,c,z] gives the Gauss's
 hypergeometric functions of second kind."

RiemannHypergeometricP::usage =
"RiemannHypergeometricP[{{a,b,c},{d,e,f},{g,h,i}},z]
 gives the Riemann's hypergeometric functions of first kind."

RiemannHypergeometricQ::usage =
"RiemannHypergeometricQ[{{a,b,c},{d,e,f},{g,h,i}},z]
 gives the Riemann's hypergeometric functions of second kind."



Begin["SpecialFunctions`GaussHypergeometric`Private`"]



Attributes[LegendreQ2]={Listable}

Attributes[LegendrePCos]={Listable}

Attributes[LegendreQCos]={Listable}

Attributes[LegendrePCosAC]={Listable}

Attributes[LegendreQCosAC]={Listable}

Attributes[LegendrePCosh]={Listable}

Attributes[LegendreQCosh]={Listable}

Attributes[LegendrePCoshAC]={Listable}

Attributes[LegendreQCoshAC]={Listable}

Attributes[ToroidalP]={Listable}

Attributes[ToroidalQ]={Listable}

Attributes[ConicalP]={Listable}

Attributes[ConicalQ]={Listable}

Attributes[ConicalPAC]={Listable}

Attributes[ConicalQAC]={Listable}

Attributes[NistConicalP]={Listable}

Attributes[NistConicalQ]={Listable}

Attributes[ChebyshevV]={Listable}

Attributes[ChebyshevTNormalized]={Listable}

Attributes[ChebyshevUNormalized]={Listable}

Attributes[ChebyshevVNormalized]={Listable}

Attributes[NistChebyshevV]={Listable}

Attributes[NistChebyshevW]={Listable}

Attributes[GegenbauerS]={Listable}

Attributes[GegenbauerD]={Listable}

Attributes[GegenbauerCNormalized]={Listable}

Attributes[GegenbauerSNormalized]={Listable}

Attributes[JacobiG]={Listable}

Attributes[JacobiQ]={Listable}

Attributes[JacobiPNormalized]={Listable}

Attributes[JacobiQNormalized]={Listable}

Attributes[JacobiQFerrers]={Listable}

Attributes[JacobiQHobson]={Listable}

Attributes[RomanovskiR]={Listable}

Attributes[ZernikeZ]={Listable}

Attributes[JacobiPCos]={Listable}

Attributes[Hypergeometric2G1]={Listable}



(************************************************************************)
(*                             Common code                              *)
(************************************************************************)

(* Analytic continuation of log(sin(t)), log(cos(t)) *)

logSin[z_]:=
  Module[{n},n=If[Im[z]>0,-1,1]*(Ceiling[Re[z]/Pi]-1);
    Log[Sin[z+n*Pi]]+n*Pi*I]/;
  (NumericQ[z])

logCos[z_]:=
  Module[{n},n=If[Im[z]>0,-1,1]*(Ceiling[Re[z]/Pi+1/2]-1);
    Log[Cos[z+n*Pi]]+n*Pi*I]/;
  (NumericQ[z])



(************************************************************************)
(*                        LegendreQ2[nu,mu,a,z]                         *)
(************************************************************************)

LegendreQ2[nu_,z_]:=LegendreQ2[nu,0,z];

LegendreQ2[nu_,mu_,z_]:=LegendreQ2[nu,mu,2,z];

LegendreQ2[nu_,mu_,a_,z_]:=
  2/Pi*LegendreQ[nu,mu,a,z]/;
  (MemberQ[{1,2,3},a] && NumericQ[nu] && NumericQ[mu] && NumericQ[z])



(**********************************************************************)
(*         Auxiliary functions for LegendrePCos, LegendrePCos         *)
(**********************************************************************)

factorSinCos[n_,nu_]:=
  2*Sign[n]*(-1)^n*Sum[(-1)^k*Sin[(2 k-1)*nu*Pi],{k,1,Abs[n]}]/;
  (NumericQ[n] && NumericQ[nu])

acHobsonCosN[nu_,th_]:=
  If[Re[th]<0,
    Floor[(th+Pi)/(2 Pi)],Ceiling[(th-Pi)/(2 Pi)]]/;
  (NumericQ[nu] && NumericQ[th])



(**********************************************************************)
(*                      LegendrePCos[nu,mu,1,th]                      *)
(**********************************************************************)

LegendrePCos[nu_,mu_,1,th_]:=
  LegendrePCos[nu,mu,2,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                      LegendreQCos[nu,mu,1,th]                      *)
(**********************************************************************)

LegendreQCos[nu_,mu_,1,th_]:=
  LegendreQCos[nu,mu,2,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                      LegendrePCos[nu,mu,2,th]                      *)
(**********************************************************************)

LegendrePCos[nu_,th_]:=
  LegendrePCos[nu,0,2,th]/;
  (NumericQ[nu] && NumericQ[th])

LegendrePCos[nu_,mu_,th_]:=
  LegendrePCos[nu,mu,2,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

LegendrePCos[nu_,mu_,2,th_]:=
  If[0<Re[th]<=Pi || (N[Re[th]]===0. && Im[th]>0),
    LegendreP[nu,mu,2,Cos[th]],
    If[Im[th]<0,
      Exp[mu*Pi*I/2]*LegendrePCos[nu,mu,3,th],
      Exp[-mu*Pi*I/2]*LegendrePCos[nu,mu,3,th]]]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                      LegendreQCos[nu,mu,2,th]                      *)
(**********************************************************************)

LegendreQCos[nu_,th_]:=
  LegendreQCos[nu,0,2,th]/;
  (NumericQ[nu] && NumericQ[th])

LegendreQCos[nu_,mu_,th_]:=
  LegendreQCos[nu,mu,2,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

LegendreQCos[nu_,mu_,2,th_]:=
  If[0<Re[th]<=Pi || (N[Re[th]]===0. && Im[th]>0),
    LegendreQ[nu,mu,2,Cos[th]],
    If[Im[th]<0,
      Exp[mu*Pi*I/2]*(Exp[-2*mu*Pi*I]*
        LegendreQCos[nu,mu,3,th]+Pi*I/2*LegendrePCos[nu,mu,3,th]),
      Exp[-mu*Pi*I/2]*
        (LegendreQCos[nu,mu,3,th]-Pi*I/2*LegendrePCos[nu,mu,3,th])]]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                      LegendrePCos[nu,mu,3,th]                      *)
(**********************************************************************)

LegendrePCos[nu_,mu_,3,th_]:=
  Module[{np,mp,tp,prec=20,n},
    np=SetPrecision[nu,prec];
    mp=SetPrecision[mu,prec];
    tp=SetPrecision[th,prec];
    n=acHobsonCosN[nu,Re[th]];
    If[Abs[Re[th]]<=Pi,
      LegendreP[nu,mu,3,Cos[th]],
      If[Im[th]<0,
        Exp[2 n*np*Pi*I]*LegendreP[np,mp,3,Cos[tp]]+
          2*I*Exp[-mp*Pi*I]*factorSinCos[n,np]/
          Gamma[-np-mp]/Gamma[np+mp+1]*LegendreQ[np,mp,3,Cos[tp]],
        Exp[-2 n*np*Pi*I]*LegendreP[np,mp,3,Cos[tp]]-
          2*I*Exp[-mp*Pi*I]*factorSinCos[n,np]/
          Gamma[-np-mp]/Gamma[np+mp+1]*LegendreQ[np,mp,3,Cos[tp]]]]]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                      LegendreQCos[nu,mu,3,th]                      *)
(**********************************************************************)

LegendreQCos[nu_,mu_,3,th_]:=
  Module[{n},
    n=acHobsonCosN[nu,Re[th]];
    If[Abs[Re[th]]<=Pi,
      LegendreQ[nu,mu,3,Cos[th]],
      If[Im[th]<0,
        Exp[-2 n*nu*Pi*I]*LegendreQ[nu,mu,3,Cos[th]],
        Exp[2 n*nu*Pi*I]*LegendreQ[nu,mu,3,Cos[th]]]]]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                     LegendrePCosAC[nu,mu,3,th]                     *)
(**********************************************************************)

LegendrePCosAC[nu_,th_]:=
  LegendrePCosAC[nu,0,3,th]/;
  (NumericQ[nu] && NumericQ[th])

LegendrePCosAC[nu_,mu_,th_]:=
  LegendrePCosAC[nu,mu,3,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

LegendrePCosAC[nu_,mu_,3,th_]:=
  Module[{np,mp,tp,prec=20,n},
    np=SetPrecision[nu,prec];
    mp=SetPrecision[mu,prec];
    tp=SetPrecision[th,prec];
    n=acHobsonCosN[nu,Re[th]];
    If[Abs[Re[th]]<=Pi,
      hobsonPCos[np,mp,3,tp],
      Exp[-2 n*np*Pi*I]*hobsonPCos[np,mp,3,tp]-
      2*I*Exp[-mp*Pi*I]*factorSinCos[n,np]/
      Gamma[-np-mp]/Gamma[np+mp+1]*hobsonQCos[np,mp,3,tp]]]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

hobsonPCos[nu_,mu_,3,th_]:=
  Module[{t},
    t=If[Re[th]>0,
      Pi-Mod[Pi-Re[th],2*Pi],Mod[Re[th]+Pi,2*Pi]-Pi]+Im[th]*I;
    If[Im[th]>0,
      LegendreP[nu,mu,3,Cos[t]],
      If[0<Re[t],
        Exp[mu*Pi*I/2]*LegendreP[nu,mu,2,Cos[t]],
        Exp[-mu*Pi*I/2]*LegendreP[nu,mu,2,Cos[t]]]]]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                     LegendreQCosAC[nu,mu,3,th]                     *)
(**********************************************************************)

LegendreQCosAC[nu_,th_]:=
  LegendreQCosAC[nu,0,3,th]/;
  (NumericQ[nu] && NumericQ[th])

LegendreQCosAC[nu_,mu_,th_]:=
  LegendreQCosAC[nu,mu,3,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

LegendreQCosAC[nu_,mu_,3,th_]:=
  Module[{n},
    n=acHobsonCosN[nu,Re[th]];
    If[Abs[Re[th]]<=Pi,
      hobsonQCos[nu,mu,3,th],
      Exp[2 n*nu*Pi*I]*hobsonQCos[nu,mu,3,th]]]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

hobsonQCos[nu_,mu_,3,th_]:=
  Module[{t},
    t=If[Re[th]>0,
      Pi-Mod[Pi-Re[th],2*Pi],Mod[Re[th]+Pi,2*Pi]-Pi]+Im[th]*I;
    If[Im[th]>0,
      LegendreQ[nu,mu,3,Cos[t]],
      If[0<Re[t],
        Exp[mu*Pi*I/2]*
          (LegendreQ[nu,mu,2,Cos[t]]+Pi*I/2*LegendreP[nu,mu,2,Cos[t]]),
        Exp[3*mu*Pi*I/2]*
          (LegendreQ[nu,mu,2,Cos[t]]-Pi*I/2*LegendreP[nu,mu,2,Cos[t]])]]]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                     LegendrePCosh[nu,mu,3,th]                      *)
(**********************************************************************)

LegendrePCosh[nu_,th_]:=
  LegendrePCosh[nu,0,3,th]/;
  (NumericQ[nu] && NumericQ[th])

LegendrePCosh[nu_,mu_,th_]:=
  LegendrePCosh[nu,mu,3,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

LegendrePCosh[nu_,mu_,3,th_]:=
  LegendrePCos[nu,mu,3,I*th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                     LegendreQCosh[nu,mu,3,th]                      *)
(**********************************************************************)

LegendreQCosh[nu_,th_]:=
  LegendreQCosh[nu,0,3,th]/;
  (NumericQ[nu] && NumericQ[th])

LegendreQCosh[nu_,mu_,th_]:=
  LegendreQCosh[nu,mu,3,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

LegendreQCosh[nu_,mu_,3,th_]:=
  LegendreQCos[nu,mu,3,I*th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                    LegendrePCoshAC[nu,mu,3,th]                     *)
(**********************************************************************)

LegendrePCoshAC[nu_,th_]:=
  LegendrePCoshAC[nu,0,3,th]/;
  (NumericQ[nu] && NumericQ[th])

LegendrePCoshAC[nu_,mu_,th_]:=
  LegendrePCoshAC[nu,mu,3,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

LegendrePCoshAC[nu_,mu_,3,th_]:=
  LegendrePCosAC[nu,mu,3,I*th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                    LegendreQCoshAC[nu,mu,3,th]                     *)
(**********************************************************************)

LegendreQCoshAC[nu_,th_]:=
  LegendreQCoshAC[nu,0,3,th]/;
  (NumericQ[nu] && NumericQ[th])

LegendreQCoshAC[nu_,mu_,th_]:=
  LegendreQCoshAC[nu,mu,3,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

LegendreQCoshAC[nu_,mu_,3,th_]:=
  LegendreQCosAC[nu,mu,3,I*th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                        ToroidalP[nu,mu,1,z]                        *)
(**********************************************************************)

ToroidalP[nu_,mu_,1,z_]:=
  ToroidalP[nu,mu,2,z]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[z])



(**********************************************************************)
(*                        ToroidalQ[nu,mu,1,z]                        *)
(**********************************************************************)

ToroidalQ[nu_,mu_,1,z_]:=
  ToroidalQ[nu,mu,2,z]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[z])



(**********************************************************************)
(*                        ToroidalP[nu,mu,2,z]                        *)
(**********************************************************************)

ToroidalP[nu_,mu_,2,z_]:=
  LegendrePCos[nu-1/2,mu,2,I*z]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[z])



(**********************************************************************)
(*                        ToroidalQ[nu,mu,2,z]                        *)
(**********************************************************************)

ToroidalQ[nu_,mu_,2,z_]:=
  LegendreQCos[nu-1/2,mu,2,I*z]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[z])



(**********************************************************************)
(*                        ToroidalP[nu,mu,3,z]                        *)
(**********************************************************************)

ToroidalP[nu_,z_]:=
  ToroidalP[nu,0,3,z]/;
  (NumericQ[nu] && NumericQ[z])

ToroidalP[nu_,mu_,z_]:=
  ToroidalP[nu,mu,3,z]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[z])

ToroidalP[nu_,mu_,3,z_]:=
  LegendrePCoshAC[nu-1/2,mu,3,z]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[z])



(**********************************************************************)
(*                        ToroidalQ[nu,mu,3,z]                        *)
(**********************************************************************)

ToroidalQ[nu_,z_]:=
  ToroidalQ[nu,0,3,z]/;
  (NumericQ[nu] && NumericQ[z])

ToroidalQ[nu_,mu_,z_]:=
  ToroidalQ[nu,mu,3,z]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[z])

ToroidalQ[nu_,mu_,3,z_]:=
  LegendreQCoshAC[nu-1/2,mu,3,z]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[z])



(**********************************************************************)
(*                        ConicalP[nu,mu,1,th]                        *)
(**********************************************************************)

ConicalP[nu_,mu_,1,th_]:=
  ConicalP[nu,mu,2,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                        ConicalQ[nu,mu,1,th]                        *)
(**********************************************************************)

ConicalQ[nu_,mu_,1,th_]:=
  ConicalQ[nu,mu,2,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                        ConicalP[nu,mu,2,th]                        *)
(**********************************************************************)

ConicalP[nu_,th_]:=
  ConicalP[nu,0,2,th]/;
  (NumericQ[nu] && NumericQ[th])

ConicalP[nu_,mu_,th_]:=
  ConicalP[nu,mu,2,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

ConicalP[nu_,mu_,2,th_]:=
  LegendrePCos[-1/2+I*nu,mu,2,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                        ConicalQ[nu,mu,2,th]                        *)
(**********************************************************************)

ConicalQ[nu_,th_]:=
  ConicalQ[nu,0,2,th]/;
  (NumericQ[nu] && NumericQ[th])

ConicalQ[nu_,mu_,th_]:=
  ConicalQ[nu,mu,2,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

ConicalQ[nu_,mu_,2,th_]:=
  LegendreQCos[-1/2+I*nu,mu,2,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                        ConicalP[nu,mu,3,th]                        *)
(**********************************************************************)

ConicalP[nu_,mu_,3,th_]:=
  LegendrePCos[-1/2+I*nu,mu,3,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                        ConicalQ[nu,mu,3,th]                        *)
(**********************************************************************)

ConicalQ[nu_,mu_,3,th_]:=
  LegendreQCos[-1/2+I*nu,mu,3,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                       ConicalPAC[nu,mu,3,th]                       *)
(**********************************************************************)

ConicalPAC[nu_,th_]:=
  ConicalPAC[nu,0,3,th]/;
  (NumericQ[nu] && NumericQ[th])

ConicalPAC[nu_,mu_,th_]:=
  ConicalPAC[nu,mu,3,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

ConicalPAC[nu_,mu_,3,th_]:=
  LegendrePCosAC[-1/2+I*nu,mu,3,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                       ConicalQAC[nu,mu,3,th]                       *)
(**********************************************************************)

ConicalQAC[nu_,th_]:=
  ConicalQAC[nu,0,3,th]/;
  (NumericQ[nu] && NumericQ[th])

ConicalQAC[nu_,mu_,th_]:=
  ConicalQAC[nu,mu,3,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])

ConicalQAC[nu_,mu_,3,th_]:=
  LegendreQCosAC[-1/2+I*nu,mu,3,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                       NistConicalP[nu,mu,th]                       *)
(**********************************************************************)

NistConicalP[nu_,th_]:=
  NistConicalP[nu,0,th]/;
  (NumericQ[nu] && NumericQ[th])

NistConicalP[nu_,mu_,th_]:=
  ConicalP[nu,mu,2,th]/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(**********************************************************************)
(*                       NistConicalQ[nu,mu,th]                       *)
(**********************************************************************)

NistConicalQ[nu_,th_]:=
  NistConicalQ[nu,0,th]/;
  (NumericQ[nu] && NumericQ[th])

NistConicalQ[nu_,mu_,th_]:=
  Pi/2*(-Exp[-nu*Pi]*Sin[mu*Pi]*Sinh[nu*Pi]*ConicalP[nu,mu,2,th]+
  (Exp[-nu*Pi]*Cos[mu*Pi]^2+Sinh[nu*Pi])*ConicalP[nu,mu,2,Pi-th])/
  (Cosh[nu*Pi]^2-Sin[mu*Pi]^2)/;
  (NumericQ[nu] && NumericQ[mu] && NumericQ[th])



(************************************************************************)
(*                           ChebyshevV[v,z]                            *)
(************************************************************************)

ChebyshevV[v_,z_]:=
  Sqrt[1-z^2] ChebyshevU[v-1,z]/;
  (NumericQ[v] && NumericQ[z])



(************************************************************************)
(*                      ChebyshevTNormalized[v,z]                       *)
(************************************************************************)

ChebyshevTNormalized[v_,z_]:=
  ChebyshevT[v,z]/Sqrt[Pi/2]/((1-z^2)^(1/4))/;
  (Chop[v]=!=0 && NumericQ[v] && NumericQ[z])

ChebyshevTNormalized[v_,z_]:=
  ChebyshevT[v,z]/Sqrt[Pi]/((1-z^2)^(1/4))/;
  (Chop[v]===0 && NumericQ[v] && NumericQ[z])



(************************************************************************)
(*                      ChebyshevUNormalized[v,z]                       *)
(************************************************************************)

ChebyshevUNormalized[v_,z_]:=
  ChebyshevU[v,z]/Sqrt[Pi/2] ((1-z^2)^(1/4))/;
  (NumericQ[v] && NumericQ[z])



(************************************************************************)
(*                      ChebyshevVNormalized[v,z]                       *)
(************************************************************************)

ChebyshevVNormalized[v_,z_]:=
  ChebyshevV[v,z]/Sqrt[Pi/2]/((1-z^2)^(1/4))/;
  (NumericQ[v] && NumericQ[z])



(************************************************************************)
(*                         NistChebyshevV[v,z]                          *)
(************************************************************************)

NistChebyshevV[v_,z_]:=
  Module[{th=ArcCos[z]},
    Sin[(v+1/2)*th]/Sin[th/2]]/;
  (NumericQ[v] && NumericQ[z])



(************************************************************************)
(*                         NistChebyshevW[v,z]                          *)
(************************************************************************)

NistChebyshevW[v_,z_]:=
  Module[{th=ArcCos[z]},
    Cos[(v+1/2)*th]/Cos[th/2]]/;
  (NumericQ[v] && NumericQ[z])



(************************************************************************)
(*                          GegenbauerS[v,w,z]                          *)
(************************************************************************)

GegenbauerS[v_,w_,z_]:=
  Module[{prec=50,np,mp,zp},
    np=SetPrecision[v,prec];
    mp=SetPrecision[w,prec];
    zp=SetPrecision[z,prec];
    (np+1)*Cos[Pi*np/2]*Binomial[(np-1)/2+mp,mp-1]*zp*
      Hypergeometric2F1[(1-np)/2,(1+np)/2+mp,3/2,zp^2]-
      Sin[Pi*np/2]*Binomial[np/2+mp-1,mp-1]*
      Hypergeometric2F1[np/2+mp,-np/2,1/2,zp^2]]/;
  (NumericQ[v] && NumericQ[w] && NumericQ[z])



(************************************************************************)
(*                          GegenbauerD[v,w,z]                          *)
(************************************************************************)

GegenbauerD[v_,w_,z_]:=
  Module[{prec=50,np,mp,zp,val},
    np=SetPrecision[v,prec];
    mp=SetPrecision[w,prec];
    zp=SetPrecision[z,prec];
    val=-I/Sqrt[Pi]*Exp[I*Pi*mp]*Gamma[mp] Gamma[np+2 mp]/Gamma[np+1]*
      ((zp+1)/2)^(1/4-mp/2)*((zp-1)/2)^(1/4-mp/2)*LegendreQ[np+mp-1/2,1/2-mp,3,zp];
    If[Abs[Im[v]]>0 || Abs[Im[w]]>0 || Abs[Im[z]]>0 || Re[z]<1,
      val,Re[val]]]/;
  ((Abs[Im[v]]>0 || Abs[v-Floor[v+1/2]]>0 || Re[v]>-0.5) && 
   NumericQ[v] && NumericQ[w] && NumericQ[z])

GegenbauerD[v_,w_,z_]:=
  Module[{prec=50,np,mp,zp,val},
    np=SetPrecision[v,prec];
    mp=SetPrecision[w,prec];
    zp=SetPrecision[z,prec];
    val=Gamma[mp]*Gamma[np+2 mp]*2^(-np-1)*zp^(-np-2 mp)*
      Hypergeometric2F1Regularized[np/2+mp,np/2+mp+1/2,np+mp+1,1/zp^2];
    If[Abs[Im[v]]>0 || Abs[Im[w]]>0 || Abs[Im[z]]>0 || Re[z]<1,
      val,Re[val]]]/;
  (Not[Abs[Im[v]]>0 || Abs[v-Floor[v+1/2]]>0 || Re[v]>-0.5] && 
   NumericQ[v] && NumericQ[w] && NumericQ[z])



(************************************************************************)
(*                     GegenbauerCNormalized[v,w,z]                     *)
(************************************************************************)

GegenbauerCNormalized[v_,w_,z_]:=
  2^w*Gamma[w]*Sqrt[(v+w)/(2*Pi)]*Exp[LogGamma[v+1]/2-LogGamma[v+2 w]/2]*
  ((1-z^2)^(w/2-1/4))*GegenbauerC[v,w,z]/;
  (NumericQ[v] && NumericQ[w] && NumericQ[z])

GegenbauerCNormalized[v_,z_]:=0/;
  (NumericQ[v] && NumericQ[z])



(************************************************************************)
(*                     GegenbauerSNormalized[v,w,z]                     *)
(************************************************************************)

GegenbauerSNormalized[v_,w_,z_]:=
  2/Pi*Sqrt[v+w]*Exp[LogGamma[v+2 w]/2-LogGamma[v+1]/2]*
  LegendreQ[w+v-1/2,1/2-w,2,z]/;
  (NumericQ[v] && NumericQ[w] && NumericQ[z])



(************************************************************************)
(*     UltraSphericalHarmonicY[{n[1],...,n[d-1]},{t[1],...,t[d-1]}]     *)
(************************************************************************)

UltraSphericalHarmonicY[n_List,t_List]:=
  Module[{d=Length[n]+1},
    Apply[Times,
      Map[SphericalHarmonicY[n[[#]]+#/2-1,-n[[#-1]]-#/2+1,t[[#]],t[[1]]]*
        Sin[t[[#]]]^(1-#/2)&,Range[2,d-1]]]*
      Exp[I*((d-2) (d-3)/4+2*n[[1]]+Total[n[[2;;d-2]]])*t[[1]]]*
      (2*Pi)^((d-3)/2)]/;
  (OrderedQ[n] && allIntQ[n] && allNumQ[t] && Length[n]===Length[t] && 
   Not[n[[1]]===0 && (Abs[t[[2]]]<=0 || Abs[t[[2]]-Pi]<=0) && allNot0OrPiQ[Drop[t,2]]] && 
   Not[(n[[1]]=!=0 && (Abs[t[[2]]]<=0 || Abs[t[[2]]-Pi]<=0) && allNot0OrPiQ[Drop[t,2]])|| 
   member0OrPiQ[Drop[t,2]]])

UltraSphericalHarmonicY[n_List,t_List]:=
  Module[{d=Length[n]+1,val},
    val=Apply[Times,
      Map[SphericalHarmonicY[n[[#]]+#/2-1,-n[[#-1]]-#/2+1,t[[#]],t[[1]]]*
        Sin[t[[#]]]^(1-#/2)&,Range[3,d-1]]]*
      Sqrt[(2*n[[2]]+1)/(4*Pi)]*
      Exp[I*((d-2) (d-3)/4+2*n[[1]]+Total[n[[2;;d-2]]])*t[[1]]]*
      (2*Pi)^((d-3)/2);
    Which[Not[Abs[t[[2]]]>0],val,Not[Abs[t[[2]]-Pi]>0],(-1)^n[[2]]*val]]/;
  (OrderedQ[n] && allIntQ[n] && allNumQ[t] && Length[n]===Length[t] && 
   n[[1]]===0 && (Abs[t[[2]]]<=0 || Abs[t[[2]]-Pi]<=0) && allNot0OrPiQ[Drop[t,2]])

UltraSphericalHarmonicY[n_List,t_List]:=
  0/;
  (OrderedQ[n] && allIntQ[n] && allNumQ[t] && Length[n]===Length[t] && 
   ((n[[1]]=!=0 && (Abs[t[[2]]]<=0 || Abs[t[[2]]-Pi]<=0) && allNot0OrPiQ[Drop[t,2]])|| 
   member0OrPiQ[Drop[t,2]]))

allIntQ[n_List]:=
  FreeQ[Map[IntegerQ[#]&,n],False];

allNumQ[t_List]:=
  FreeQ[Map[NumericQ[#]&,t],False];

allNot0OrPiQ[t_List]:=
  FreeQ[Map[Abs[#]>0 && Abs[#-Pi]>0&,t],False];

member0OrPiQ[t_List]:=
  MemberQ[Map[Abs[#]>0 && Abs[#-Pi]>0&,t],False];



(************************************************************************)
(*                           JacobiG[v,p,q,z]                           *)
(************************************************************************)

JacobiG[v_,p_,q_,z_]:=
  JacobiP[v,p-q,q-1,2*z-1]/Binomial[2*v+p-1,v]/;
  (NumericQ[v] && NumericQ[p] && NumericQ[q] && NumericQ[z])



(************************************************************************)
(*                           JacobiQ[v,a,b,z]                           *)
(************************************************************************)

JacobiQ[v_,a_,b_,z_]:=
  Module[{prec=20,vp,ap,bp,zp},
    Block[{$MinPrecision=prec},
      vp=SetPrecision[v,prec];
      ap=SetPrecision[a,prec];
      bp=SetPrecision[b,prec];
      zp=SetPrecision[z,prec];
      ((1-zp)/2)^(-ap)*
       Gamma[vp+bp+1]*Gamma[ap]/Gamma[vp+ap+bp+1]/Pi*
       Hypergeometric2F1[-vp-ap,vp+bp+1,1-ap,(1-zp)/2]+
       Cos[ap*Pi]*Gamma[vp+ap+1]*Gamma[-ap]/Gamma[vp+1]/Pi*
       Hypergeometric2F1[-vp,vp+ap+bp+1,1+ap,(1-zp)/2]]]/;
  (Not[Not[Abs[Im[a]]>0] && Chop[a-Floor[a+1/2]]===0] && 
  (Not[Abs[Im[v]]>0] && Chop[v-1/2-Floor[v]]===0 && Re[z]<=-1) && 
   NumericQ[v] && NumericQ[a] && NumericQ[b] && NumericQ[z])

JacobiQ[v_,a_,b_,z_]:=
  ((1-z)/2)^(-a)*
  Gamma[v+b+1]*Gamma[a]/Gamma[v+a+b+1]/Pi*
  Hypergeometric2F1[-v-a,v+b+1,1-a,(1-z)/2]+
  Cos[a*Pi]*Gamma[v+a+1]*Gamma[-a]/Gamma[v+1]/Pi*
  Hypergeometric2F1[-v,v+a+b+1,1+a,(1-z)/2]/;
  (Not[Not[Abs[Im[a]]>0] && Chop[a-Floor[a+1/2]]===0] && 
   Not[Not[Abs[Im[v]]>0] && Chop[v-1/2-Floor[v]]===0 && Re[z]<=-1] && 
   NumericQ[v] && NumericQ[a] && NumericQ[b] && NumericQ[z])

JacobiQ[v_,a_,b_,z_]:=
  2*(JacobiQ[v+1,a-1,b-1,z]-JacobiQ[v+1,a-1,b,z])/(1-z)/;
  (Re[a]>0.9 && Not[Abs[Im[a]]>0] && Chop[a-Floor[a+1/2]]===0 && 
   NumericQ[v] && NumericQ[a] && NumericQ[b] && NumericQ[z])

JacobiQ[v_,a_,b_,z_]:=
  JacobiQ[v,a+1,b,z]-(z+1)/2*JacobiQ[v-1,a+1,b+1,z]/;
  (Re[a]<-0.9 && Not[Abs[Im[a]]>0] && Chop[a-Floor[a+1/2]]===0 && 
   NumericQ[v] && NumericQ[a] && NumericQ[b] && NumericQ[z])

JacobiQ[v_,a_,b_,z_]:=
  -(Hypergeometric2F1[-v,1+b+v,1,(1-z)/2]*
    (HarmonicNumber[v]+HarmonicNumber[v+b]+Log[(1-z)/2])+
    2*hypgeo2F1D0010[-v,v+b+1,1,(1-z)/2]+hypgeo2F1D0100[-v,v+b+1,1,(1-z)/2]+
    hypgeo2F1D1000[-v,v+b+1,1,(1-z)/2])/Pi/;
  (Abs[Re[a]]<0.1 && Not[Abs[Im[a]]>0] && Chop[a-Floor[a+1/2]]===0 && 
   Not[Abs[Im[v]]>0 || Abs[Im[b]]>0 || Abs[Im[z]]>0] && 
   NumericQ[v] && NumericQ[a] && NumericQ[b] && NumericQ[z])

hypgeo2F1D1000[a_,b_,c_,z_]:=
  \!\(\*
TagBox[
StyleBox[
RowBox[{
RowBox[{
RowBox[{"Derivative", "[", 
RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "]"}], "[", "Hypergeometric2F1", "]"}], "[", 
RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}],
ShowSpecialCharacters->False,
ShowStringCharacters->True,
NumberMarks->True],
FullForm]\);

hypgeo2F1D0100[a_,b_,c_,z_]:=
  \!\(\*
TagBox[
StyleBox[
RowBox[{
RowBox[{
RowBox[{"Derivative", "[", 
RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "]"}], "[", "Hypergeometric2F1", "]"}], "[", 
RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}],
ShowSpecialCharacters->False,
ShowStringCharacters->True,
NumberMarks->True],
FullForm]\);

hypgeo2F1D0010[a_,b_,c_,z_]:=\!\(\*
TagBox[
StyleBox[
RowBox[{"\n", "  ", 
RowBox[{
RowBox[{
RowBox[{"Derivative", "[", 
RowBox[{"0", ",", "0", ",", "1", ",", "0"}], "]"}], "[", "Hypergeometric2F1", "]"}], "[", 
RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}]}],
ShowSpecialCharacters->False,
ShowStringCharacters->True,
NumberMarks->True],
FullForm]\);

JacobiQ[v_,a_,b_,z_]:=
  (JacobiP[v,0,b,z]*(HarmonicNumber[v]-HarmonicNumber[v+b]+Log[2]-Log[1-z])+
    JacobiPD1000[v,0,b,z]-2*JacobiPD0100[v,0,b,z])/Pi/;
  (Abs[Re[a]]<0.1 && Not[Abs[Im[a]]>0] && Chop[a-Floor[a+1/2]]===0 && 
   (Abs[Im[v]]>0 || Abs[Im[b]]>0 || Abs[Im[z]]>0) && 
   NumericQ[v] && NumericQ[a] && NumericQ[b] && NumericQ[z])

JacobiPD1000[v_,a_,b_,z_]:=
  \!\(\*
TagBox[
StyleBox[
RowBox[{
RowBox[{
RowBox[{"Derivative", "[", 
RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "]"}], "[", "JacobiP", "]"}], "[", 
RowBox[{"v", ",", "a", ",", "b", ",", "z"}], "]"}],
ShowSpecialCharacters->False,
ShowStringCharacters->True,
NumberMarks->True],
FullForm]\);

JacobiPD0100[v_,a_,b_,z_]:=
\!\(\*
TagBox[
StyleBox[
RowBox[{"  ", 
RowBox[{
RowBox[{
RowBox[{"Derivative", "[", 
RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "]"}], "[", "JacobiP", "]"}], "[", 
RowBox[{"v", ",", "a", ",", "b", ",", "z"}], "]"}]}],
ShowSpecialCharacters->False,
ShowStringCharacters->True,
NumberMarks->True],
FullForm]\);



(************************************************************************)
(*                      JacobiPNormalized[v,a,b,z]                      *)
(************************************************************************)

JacobiPNormalized[v_,a_,b_,z_]:=
  Sqrt[2*v+a+b+1]/(2^((a+b+1)/2))*
  Exp[(LogGamma[v+1]+LogGamma[v+a+b+1]-LogGamma[v+a+1]-LogGamma[v+b+1])/2]*
  (1-z)^(a/2)*(1+z)^(b/2)*JacobiP[v,a,b,z]/;
  (NumericQ[v] && NumericQ[a] && NumericQ[b] && NumericQ[z])



(************************************************************************)
(*                      JacobiQNormalized[v,a,b,z]                      *)
(************************************************************************)

JacobiQNormalized[v_,a_,b_,z_]:=
  Sqrt[2*v+a+b+1]/(2^((a+b+1)/2))*
  Exp[(LogGamma[v+1]+LogGamma[v+a+b+1]-LogGamma[v+a+1]-LogGamma[v+b+1])/2]*
  (1-z)^(a/2)*(1+z)^(b/2)*JacobiQ[v,a,b,z]/;
  (NumericQ[v] && NumericQ[a] && NumericQ[b] && NumericQ[z])



(************************************************************************)
(*                       JacobiQFerrers[v,a,b,z]                        *)
(************************************************************************)

JacobiQFerrers[v_,a_,b_,z_]:=
  2^(a+b-1)*Cos[a*Pi]/(a*Binomial[v+a+b,a])*(1-z)^(-a)*(1+z)^(-b)*
  Hypergeometric2F1[v+1,-v-a-b,1-a,(1-z)/2]-Pi/2*Csc[a*Pi]*JacobiP[v,a,b,z]/;
  (Not[Chop[Im[a]]===0 && Chop[a-Floor[a+1/2]]===0] && 
   NumericQ[v] && NumericQ[a] && NumericQ[b] && NumericQ[z])

JacobiQFerrers[v_,a_,b_,z_]:=
  Module[{prec=20,vp,ap,bp,zp},
    Block[{$MinPrecision=prec},
      vp=SetPrecision[v,prec];
      ap=SetPrecision[a,prec];
      bp=SetPrecision[b,prec];
      zp=SetPrecision[z,prec];
      -Pi/2*(Sin[ap*Pi]*JacobiP[vp,ap,bp,zp]-Cos[ap*Pi]*JacobiQ[vp,ap,bp,zp])]]/;
  (Chop[Im[a]]===0 && Chop[a-Floor[a+1/2]]===0 && 
   NumericQ[v] && NumericQ[a] && NumericQ[b] && NumericQ[z])



(************************************************************************)
(*                        JacobiQHobson[v,a,b,z]                        *)
(************************************************************************)

JacobiQHobson[v_,a_,b_,z_]:=
  2^(v+a+b)/((v+b+1)*Binomial[2*v+a+b+1,v+a])*(z-1)^(-v-a-1)*(z+1)^(-b)*
  Hypergeometric2F1[v+1,v+a+1,2*v+a+b+2,2/(1-z)]/;
  (NumericQ[v] && NumericQ[a] && NumericQ[b] && NumericQ[z])



(************************************************************************)
(*                         RomanovskiR[v,a,b,z]                         *)
(************************************************************************)

RomanovskiR[v_,a_,b_,z_]:=
  I^v*JacobiP[v,b-1+I*a/2,b-1-I*a/2,I*z]/;
  (NumericQ[v] && NumericQ[a] && NumericQ[b] && NumericQ[z])



(************************************************************************)
(*                        ZernikeZ[n,m,rho,phi]                         *)
(************************************************************************)

ZernikeZ[n_,m_,rho_,phi_]:=
  If[Re[m]>=0,
    ZernikeR[n,m,rho]*Cos[m*phi],-ZernikeR[n,-m,rho]*Sin[m*phi]]/;
  (NumericQ[n] && NumericQ[m] && NumericQ[rho] && NumericQ[phi])



(************************************************************************)
(*                        JacobiPCos[nu,a,b,th]                         *)
(************************************************************************)

JacobiPCos[nu_,alp_,bet_,the_]:=
  If[Abs[Re[the]]<Pi,JacobiP[nu,alp,bet,Cos[the]],
    If[Pi<=Abs[Re[the]]<2*Pi,jacobiPCos1[nu,alp,bet,the],
      If[Im[the]<0 || Not[Abs[Im[the]]>0] && Re[the]>0,
        jacobiPCos2[nu,alp,bet,the],jacobiPCos2[nu,alp,bet,the+2*Pi]]]]/;
  (NumericQ[nu] && NumericQ[alp] && NumericQ[bet] && NumericQ[the])

jacobiPCos1[nu_,alp_,bet_,the_]:=
  Binomial[nu+alp,nu]*hypergeom2F1Ana1[-nu,nu+alp+bet+1,alp+1,the];

hypergeom2F1Ana1[a_,b_,c_,the_]:=
  Gamma[c]*Gamma[c-a-b]/Gamma[c-a]/Gamma[c-b]*
    Hypergeometric2F1[a,b,a+b-c+1,Cos[the/2]^2]+
  Gamma[c]*Gamma[a+b-c]/Gamma[a]/Gamma[b]*Cos[the/2]^(2*(c-a-b))*
    Hypergeometric2F1[c-a,c-b,c-a-b+1,Cos[the/2]^2];

jacobiPCos2[nu_,alp_,bet_,the_]:=
  Binomial[nu+alp,nu]*hypergeom2F1Ana2[-nu,nu+alp+bet+1,alp+1,the];

hypergeom2F1Ana2[a_,b_,c_,the_]:=
  Gamma[c]*Gamma[b-a]/Gamma[b]/Gamma[c-a]*(-1)^(-a)*Exp[-2*a*logSin[the/2]]*
    Hypergeometric2F1[a,a-c+1,a-b+1,Csc[the/2]^2]+
  Gamma[c]*Gamma[a-b]/Gamma[a]/Gamma[c-b]*(-1)^(-b)*Exp[-2*b*logSin[the/2]]*
    Hypergeometric2F1[b,b-c+1,b-a+1,Csc[the/2]^2];



(************************************************************************)
(*                   Wigner's D-function, d-function                    *)
(************************************************************************)

(* Wigner's D-functions in ordinary definition *)

WignerDOrdinary[{j_,m1_,m2_},psi_,the_,phi_]:=
  Exp[-I*(m1*psi+m2*phi)]*WignerDOrdinary[{j,m1,m2},the];

WignerDOrdinary[{j_,m1_,m2_},the_]:=
  WignerD[{j,-m1,-m2},the]/phaseMathDef[m1,m2];

(* Analytic continuation of Wigner's D-functions in ordinary definition *)

WignerDOrdinaryAnalytic[{j_,m1_,m2_},psi_,the_,phi_]:=
  Exp[-I*(m1*psi+m2*phi)]*WignerDOrdinaryAnalytic[{j,m1,m2},the];

WignerDOrdinaryAnalytic[{j_,m1_,m2_},the_]:=
  analyticContinuWD[{j,m1,m2},the];

(* Analytic continuation of Wigner's D-functions in Mathematica definition *)

WignerDAnalytic[{j_,m1_,m2_},psi_,the_,phi_]:=
  Exp[I*(m1*psi+m2*phi)]*WignerDAnalytic[{j,m1,m2},the];

WignerDAnalytic[{j_,m1_,m2_},the_]:=
  phaseMathDef[-m1,-m2]*analyticContinuWD[{j,-m1,-m2},the];

(******************* Sub code *******************)

(* Phase factor in Mathematica definition *)

phaseMathDef[m1_,m2_]:=
  If[Re[m1-m2]>0,(1+Exp[-2*Pi*I*(m1-m2)])/2,1];

(* Analytic continuation *)

analyticContinuWD[{j_,m1_,m2_},the_]:=
  Module[{kL,kMin,k,kPos,a,b,lam},
    kL={j+m2,j-m2,j+m1,j-m1};
    k=Sort[kL][[1]];
    kPos=Position[kL,k][[1,1]];
    a={m1-m2,m2-m1}[[Ceiling[Abs[2/(5-2*kPos)]]]];
    b={-m1-m2,m1+m2}[[Mod[kPos,2,1]]];
    lam={m1-m2,0}[[Ceiling[Abs[2/(5-2*kPos)]]]];
      (-1)^lam*Exp[(LogGamma[2*j-k+1]+LogGamma[b+1]+LogGamma[k+1]-
      LogGamma[k+a+1]-LogGamma[2*j-2*k-a+1]-LogGamma[k+b+1])/2]*
      Exp[a*logSin[the/2]]*Exp[b*logCos[the/2]]*JacobiPCos[k,a,b,the]];



(************************************************************************)
(*                      Hypergeometric2G1[a,b,c,z]                      *)
(************************************************************************)

Hypergeometric2G1[a_,b_,c_,z_]:=
  Gamma[1-a]*Gamma[b-c+1]*Gamma[c]*Gamma[c-1]/
  (Gamma[b]*Gamma[c-a]*Pi)*
  Hypergeometric2F1[a-c+1,b-c+1,2-c,z]*z^(1-c)-
  Cot[c*Pi]*Hypergeometric2F1[a,b,c,z]/;
  (NumericQ[a] && NumericQ[b] && NumericQ[c] && NumericQ[z])



(************************************************************************)
(*                        RiemannHypergeometricP                        *)
(************************************************************************)

RiemannHypergeometricP[{{a_,b_,c_},{d_,e_,f_},{g_,h_,i_}},z_]:=
  Module[{normal},
    normal=outsideRHP[{{a,b,c},{d,e,f},{g,h,i}},0.]/
           insideRHP[{{a,b,c},{d,e,f},{g,h,i}},0.];
    If[Abs[z-circleCen[{a,b,c}]]>circleRad[{a,b,c}],
      outsideRHP[{{a,b,c},{d,e,f},{g,h,i}},z],
      normal*insideRHP[{{a,b,c},{d,e,f},{g,h,i}},z]]]/;
  (NumericQ[z] && Not[IntegerQ[d-g]] && Not[IntegerQ[e-h]] && 
   Not[IntegerQ[f-i]] && N[d+e+f+g+h+i]==1)

outsideRHP[{{a_,b_,c_},{d_,e_,f_},{g_,h_,i_}},z_]:=
  Hypergeometric2F1[d+e+f,d+h+f,1+d-g,(z-a) (c-b)/((z-b) (c-a))]*
  ((z-a)/(z-b))^d*((z-c)/(z-b))^f;

insideRHP[{{a_,b_,c_},{d_,e_,f_},{g_,h_,i_}},z_]:=
  outsideRHP[{{transRHP[{a,b,c},a],transRHP[{a,b,c},b],transRHP[{a,b,c},c]},
  {d,e,f},{g,h,i}},transRHP[{a,b,c},z]];

transRHP[{a_,b_,c_},z_]:=
  circleRad[{a,b,c}]^2/(z-circleCen[{a,b,c}])+circleCen[{a,b,c}];

circleCen[{a_,b_,c_}]:=
  (Im[a (b-c) Conjugate[a]+b (c-a) Conjugate[b]+(a-b) c Conjugate[c]]+
  I*(c Conjugate[c] Re[b-a]+Abs[b]^2*Re[a-c]+Abs[a]^2*Re[c-b]))/
  Re[(c-b) Im[a]+(a-c) Im[b]+(b-a) Im[c]]/2;

circleRad[{a_,b_,c_}]:=Abs[a-circleCen[{a,b,c}]];



(************************************************************************)
(*                        RiemannHypergeometricQ                        *)
(************************************************************************)

RiemannHypergeometricQ[{{a_,b_,c_},{d_,e_,f_},{g_,h_,i_}},z_]:=
  Module[{normal},
    normal=outsideRHQ[{{a,b,c},{d,e,f},{g,h,i}},0.]/
           insideRHQ[{{a,b,c},{d,e,f},{g,h,i}},0.];
    If[Abs[z-circleCen[{a,b,c}]]>circleRad[{a,b,c}],
      outsideRHQ[{{a,b,c},{d,e,f},{g,h,i}},z],
      normal*insideRHQ[{{a,b,c},{d,e,f},{g,h,i}},z]]]/;
  (NumericQ[z] && Not[IntegerQ[d-g]] && Not[IntegerQ[e-h]] && 
   Not[IntegerQ[f-i]] && N[d+e+f+g+h+i]==1)

outsideRHQ[{{a_,b_,c_},{d_,e_,f_},{g_,h_,i_}},z_]:=
  Hypergeometric2G1[d+e+f,d+h+f,1+d-g,(z-a) (c-b)/((z-b) (c-a))]*
  ((z-a)/(z-b))^d*((z-c)/(z-b))^f;

insideRHQ[{{a_,b_,c_},{d_,e_,f_},{g_,h_,i_}},z_]:=
  outsideRHQ[{{transRHP[{a,b,c},a],transRHP[{a,b,c},b],transRHP[{a,b,c},c]},
  {d,e,f},{g,h,i}},transRHP[{a,b,c},z]];



(************************************************************************)

End[]

Protect[LegendreQ2, LegendrePCos, LegendreQCos, 
   LegendrePCosAC, LegendreQCosAC, 
   LegendrePCosh, LegendreQCosh, LegendrePCoshAC, LegendreQCoshAC, 
   ToroidalP, ToroidalQ, ConicalP, ConicalQ, 
   ConicalPAC, ConicalQAC, NistConicalP, NistConicalQ, 
   ChebyshevV, ChebyshevTNormalized, ChebyshevUNormalized, 
   ChebyshevVNormalized, NistChebyshevV, NistChebyshevW, 
   GegenbauerS, GegenbauerD, GegenbauerCNormalized, 
   GegenbauerSNormalized, UltraSphericalHarmonicY, 
   JacobiG, JacobiQ, JacobiPNormalized, JacobiQNormalized, 
   JacobiQFerrers, JacobiQHobson, RomanovskiR, ZernikeZ, 
   JacobiPCos, WignerDOrdinary, WignerDOrdinaryAnalytic, WignerDAnalytic, 
   Hypergeometric2G1, RiemannHypergeometricP, 
   RiemannHypergeometricQ]

EndPackage[]
